use std::convert::TryFrom;
use std::io::{stdout, IsTerminal};
use std::path::PathBuf;

use anyhow::Result;
use clap::{crate_description, crate_name, crate_version, value_parser};
use clap::{Arg, ArgAction, ArgGroup, ArgMatches, Command};

pub enum Mode {
    Balance,
    ListCredit,
    ListDebit,
}

#[derive(Clone, Copy, Debug)]
pub enum Color {
    Yes,
    No,
    Always,
}

impl Color {
    pub fn with_interactive(self) -> bool {
        match self {
            Self::Yes => stdout().is_terminal(),
            Self::No => false,
            Self::Always => true,
        }
    }
}

pub struct Arguments {
    pub mode: Mode,
    pub color: Color,
    pub gleeo: Option<PathBuf>,
    pub sqlite: Option<PathBuf>,
}

impl TryFrom<ArgMatches> for Arguments {
    type Error = anyhow::Error;

    fn try_from(args: ArgMatches) -> Result<Self, Self::Error> {
        let mode = if args.get_flag("list-credit") {
            Mode::ListCredit
        } else if args.get_flag("list-debit") {
            Mode::ListDebit
        } else {
            Mode::Balance
        };

        let color = match args.get_one::<String>("color").unwrap().as_str() {
            "yes" => Color::Yes,
            "always" => Color::Always,
            "no" => Color::No,
            _ => unreachable!("clap::Arg::possible_values"),
        };

        let gleeo = args.get_one::<PathBuf>("gleeo").map(ToOwned::to_owned);
        let sqlite = args.get_one::<PathBuf>("sqlite").map(ToOwned::to_owned);

        Ok(Self {
            mode,
            color,
            gleeo,
            sqlite,
        })
    }
}

pub fn args() -> Result<Arguments> {
    let cli = build();
    let args = cli.get_matches();
    let arguments = Arguments::try_from(args)?;

    Ok(arguments)
}

pub fn build() -> Command {
    let list_credit = Arg::new("list-credit")
        .long("list-credit")
        .action(ArgAction::SetTrue)
        .help("list credit");

    let list_debit = Arg::new("list-debit")
        .long("list-debit")
        .action(ArgAction::SetTrue)
        .help("list debit");

    let mode = ArgGroup::new("mode").arg("list-credit").arg("list-debit");

    let color = Arg::new("color")
        .long("color")
        .help("control ANSI color escapes")
        .value_parser(clap::builder::PossibleValuesParser::new([
            "yes", "no", "always",
        ]))
        .default_value("yes");

    let sqlite = Arg::new("sqlite")
        .long("debit")
        .help("sqlite debit file")
        .value_parser(value_parser!(PathBuf))
        .required_unless_present("list-credit");

    let gleeo = Arg::new("gleeo")
        .long("gleeo")
        .help("gleeo credit file")
        .value_parser(value_parser!(PathBuf))
        .required_unless_present("list-debit");

    let help = Arg::new("help")
        .short('?')
        .long("help")
        .help("print help (use --help to see all options)")
        .long_help("Print help.")
        .action(ArgAction::Help);

    let version = Arg::new("version")
        .long("version")
        .long_help("Print version.")
        .hide_short_help(true)
        .action(ArgAction::Version);

    Command::new(crate_name!())
        .about(crate_description!())
        .version(crate_version!())
        .arg(list_credit)
        .arg(list_debit)
        .group(mode)
        .arg(color)
        .arg(sqlite)
        .arg(gleeo)
        .disable_help_flag(true)
        .disable_version_flag(true)
        .arg(help)
        .arg(version)
}
