pub mod gleeo;
pub mod timew;

use std::collections::BTreeMap;
use std::path::Path;

use anyhow::Result;
use chrono::{Datelike, IsoWeek, NaiveDate};
use prettytable::format::FormatBuilder;
use prettytable::{row, Table};

use crate::cli::Color;

#[derive(Debug)]
pub struct Credit {
    entries: Vec<Entry>,
}

impl Credit {
    pub fn weekly(&self) -> BTreeMap<IsoWeek, i64> {
        let mut data = BTreeMap::default();

        for entry in &self.entries {
            let week = entry.date.iso_week();
            *data.entry(week).or_insert(0) += entry.seconds;
        }

        data
    }
}

#[derive(Debug)]
pub struct Entry {
    date: NaiveDate,
    seconds: i64,
}

impl Entry {
    pub const fn date(&self) -> &NaiveDate {
        &self.date
    }

    pub const fn seconds(&self) -> i64 {
        self.seconds
    }
}

pub fn all<P>(gleeo: P) -> Result<Credit>
where
    P: AsRef<Path>,
{
    let mut entries = vec![];

    let gleeo = gleeo::from_path(gleeo.as_ref())?;
    let timew = timew::from_timew_export()?;

    entries.extend(gleeo);
    entries.extend(timew);

    Ok(Credit { entries })
}

pub fn print<P>(gleeo: P, color: Color) -> Result<()>
where
    P: AsRef<Path>,
{
    let credit = all(gleeo)?;

    let mut table = Table::new();
    let format = FormatBuilder::new().column_separator(' ').build();
    table.set_format(format);
    table.set_titles(row![bu->"Date", bu->"Seconds"]);

    for entry in credit.entries {
        table.add_row(row![entry.date(), r->entry.seconds()]);
    }

    let _ignored = table.print_tty(color.with_interactive());

    Ok(())
}
