pub mod sqlite;

use std::collections::BTreeMap;
use std::path::Path;

use anyhow::Result;
use chrono::naive::NaiveDate;
use chrono::{Datelike, IsoWeek};
use prettytable::format::FormatBuilder;
use prettytable::{row, Table};

use crate::cli::Color;

pub struct Debit {
    entries: Vec<Entry>,
}

impl Debit {
    pub fn weekly(&self) -> BTreeMap<IsoWeek, f64> {
        let mut data = BTreeMap::default();

        for entry in &self.entries {
            let week = entry.date.iso_week();
            *data.entry(week).or_insert(0.0) += entry.hours;
        }

        data
    }
}

#[derive(Debug)]
pub struct Entry {
    date: NaiveDate,
    hours: f64,
}

impl Entry {
    pub const fn date(&self) -> &NaiveDate {
        &self.date
    }

    pub const fn hours(&self) -> f64 {
        self.hours
    }
}

pub fn all<P>(sqlite: P) -> Result<Debit>
where
    P: AsRef<Path>,
{
    let entries = sqlite::from_path(sqlite)?;
    Ok(Debit { entries })
}

pub fn print<P>(path: P, color: Color) -> Result<()>
where
    P: AsRef<Path>,
{
    let debit = all(path)?;

    let mut table = Table::new();
    let format = FormatBuilder::new().column_separator(' ').build();
    table.set_format(format);
    table.set_titles(row![bu->"Date", bu->"Hours"]);

    for entry in debit.entries {
        table.add_row(row![entry.date(), r->entry.hours()]);
    }

    let _ignored = table.print_tty(color.with_interactive());

    Ok(())
}
